#!/usr/bin/env python3 
import pickle,math,matplotlib,sys,os,string,shutil
from sys import platform as sys_pf
if sys_pf == 'darwin':
    import matplotlib
    matplotlib.use("TkAgg")
from matplotlib import pyplot as plt
import numpy as np
import pandas as pd
pd.options.mode.chained_assignment = None
import seaborn as sns
import tkinter as tk
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
from matplotlib.widgets import RectangleSelector
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
from plateypus.dataprocessing.miscFunctions import rainbow_text

idx = pd.IndexSlice
dataTypeObservableRangeDict = {'cyt':1,'cell':3,'prolif':1}
realDataTypeNameDict = {'cyt':'Supernatant','cell':'Surface/Intracellular Marker','prolif':'Proliferation'}
plateRowLetters = string.ascii_uppercase[:16]
plateColumnNumbers = list(range(1,25))

colwrap = 6 
#For macbook by itself
#figLengthScaling = 0.5
#For work monitor
figLengthScaling = 1
#For home monitor
#figLengthScaling = 0.75

#https://stackoverflow.com/questions/16856788/slice-2d-array-into-smaller-2d-arrays
def blockshaped(arr, nrows, ncols):
    """
    Return an array of shape (n, nrows, ncols) where
    n * nrows * ncols = arr.size

    If arr is a 2D array, the returned array should look like n subblocks with
    each subblock preserving the "physical" layout of arr.
    """
    h, w = arr.shape
    assert h % nrows == 0, "{} rows is not evenly divisble by {}".format(h, nrows)
    assert w % ncols == 0, "{} cols is not evenly divisble by {}".format(w, ncols)
    return (arr.reshape(h//nrows, nrows, -1, ncols)
               .swapaxes(1,2)
               .reshape(-1, nrows, ncols))

def returnBaseLayout(plateDimensions,conditionPlateRows,timepointPlateColumns):
    if conditionPlateRows == 1 and timepointPlateColumns > colwrap:
        colwrapBool = True
    else:
        colwrapBool = False

    if colwrapBool:
        visualNumRowPlates = math.ceil(timepointPlateColumns/colwrap)
        visualNumColumnPlates = colwrap
    else:
        visualNumRowPlates = conditionPlateRows
        visualNumColumnPlates = timepointPlateColumns

    baseLayoutX = list(range(1,plateDimensions[1]*visualNumColumnPlates+1))
    baseLayoutY = list(range(plateDimensions[0]*visualNumRowPlates,0,-1))
    
    if colwrapBool:
        rowLimits = list(range((visualNumRowPlates-1)*plateDimensions[0],len(baseLayoutY)))
        plateOverhang = timepointPlateColumns - ((visualNumRowPlates-1)*colwrap)
        colLimits = list(range((plateOverhang*plateDimensions[1]),(visualNumColumnPlates*plateDimensions[1])))
    else:
        rowLimits = []
        colLimits = []

    #Slightly shift points over every time plate changes
    hshift = np.repeat(list(range(visualNumColumnPlates)),plateDimensions[1])
    vshift = np.repeat(list(range(visualNumRowPlates)),plateDimensions[0])

    baseLayoutX = np.add(np.array(baseLayoutX),np.array(hshift))
    baseLayoutY = np.add(np.array(baseLayoutY),np.array(vshift[::-1]))

    pointList = []
    infoList = []
    trueCol = -1

    for row,y in enumerate(baseLayoutY):
        if row % plateDimensions[0] == 0:
            trueCol+=1
        for col,x in enumerate(baseLayoutX):
            rowLetter = plateRowLetters[row%plateDimensions[0]]
            columnNumber = plateColumnNumbers[col%plateDimensions[1]]
            key=-1
            if not colwrapBool:
                plateName = plateRowLetters[int(row/plateDimensions[0])]+str(int(col/plateDimensions[1])+1)
            else:
                plateName = plateRowLetters[0]+str(int((trueCol*colwrap*plateDimensions[1]+col)/plateDimensions[1])+1)
            if row not in rowLimits or col not in colLimits:            
                if row % plateDimensions[0] == 0 and col % plateDimensions[1] == 0:
                    infoList.append([rowLetter+str(columnNumber),plateName,plateName])
                else:
                    infoList.append([rowLetter+str(columnNumber),'DoNotLabel',plateName])
                pointList.append([x,y,key,int(row/plateDimensions[0]),int(col/plateDimensions[1]),0])
            else:
                infoList.append([rowLetter+str(columnNumber),'DoNotLabel','None'])
                pointList.append([x,y,key,int(row/plateDimensions[0]),int(col/plateDimensions[1]),-1])

    pointMatrix = np.matrix(pointList)
    infoMatrix = np.matrix(infoList)

    if colwrapBool:
        for i,matrix in enumerate([pointMatrix,infoMatrix]):
            newList = []
            for col in range(matrix.shape[1]):
                vals = np.reshape(matrix[:,col],(visualNumRowPlates*plateDimensions[0],visualNumColumnPlates*plateDimensions[1]))
                reshapedVals = blockshaped(np.asarray(vals), plateDimensions[0], plateDimensions[1])
                reshapedValList = [reshapedVals[x,:,:] for x in range(timepointPlateColumns)]
                #reshapedValList = [reshapedVals[x,:,:] for x in range(reshapedVals.shape[0])]
                reshapedMatrix = np.hstack(reshapedValList)
                newList.append(reshapedMatrix.flatten())
            if i == 0:
                pointMatrix = np.matrix(newList).T
            else:
                infoMatrix = np.matrix(newList).T
    
    pointDf = pd.DataFrame(pointMatrix,columns=['x','y','key','plateRow','plateColumn','blank'])
    infoDf = pd.DataFrame(infoMatrix,columns=['wellID','plateID','plateName'])
    
    #Make lines to separate plates
    vlineList = []
    for vline in range(1,visualNumColumnPlates):
        vlinex = (baseLayoutX[vline*plateDimensions[1]-1]+baseLayoutX[vline*plateDimensions[1]])/2.0
        vlineList.append(vlinex)
    hlineList = []
    for hline in range(1,visualNumRowPlates):
        hliney = (baseLayoutY[hline*plateDimensions[0]-1]+baseLayoutY[hline*plateDimensions[0]])/2.0
        hlineList.append(hliney)
    
    return pointDf,infoDf,vlineList,hlineList

class BlankSelectionPage(tk.Frame):
    def __init__(self, master,folderName,levels,levelValues,maxNumLevelValues,numPlates,plateDimensions,dtl,shp,bPage):
        numRowPlates = 1
        numColumnPlates = numPlates
        if numRowPlates == 1 and numColumnPlates > colwrap:
            colwrapBool = True
        else:
            colwrapBool = False

        self.root = master.root
        tk.Frame.__init__(self, master)
        
        global secondaryhomepage,dataTypeList,backPage
        secondaryhomepage = shp
        dataTypeList = dtl
        backPage = bPage

        tk.Label(self,text='Blank Selection Page:',font=('Arial',20)).grid(row=0,column=0)

        plotFrame = tk.Frame(self)
        plotFrame.grid(row=5,column=0)
        
        if plateDimensions[1] == 24:
            scalingFactor = 1.5
        else:
            scalingFactor = 1
        
        if colwrapBool:
            visualNumRowPlates = math.ceil(numColumnPlates/colwrap)
            visualNumColumnPlates = colwrap
        else:
            visualNumRowPlates = numRowPlates
            visualNumColumnPlates = numColumnPlates 
        fig = plt.figure(figsize=(3*visualNumColumnPlates*(plateDimensions[0]/8), 2*visualNumRowPlates*(plateDimensions[1]/12)*figLengthScaling),tight_layout=True)
        gs = fig.add_gridspec(1, 1)
        fig_ax1 = fig.add_subplot(gs[0])

        baseLayoutDf,infoDf,vlinelist,hlinelist = returnBaseLayout(plateDimensions,numRowPlates,numColumnPlates)

        self.currentLayout = baseLayoutDf.copy()
        def modifyPlatePlot():
            #Remove all axis elements
            fig_ax1.set_xlim((0, max(baseLayoutDf['x'])+1))
            fig_ax1.set_ylim((0, max(baseLayoutDf['y'])+1))
            fig_ax1.set_xticks([])
            fig_ax1.set_yticks([])
            fig_ax1.set_xlabel('')
            fig_ax1.set_ylabel('')
            #Add plate dividing lines
            for vlinex in vlinelist:
                fig_ax1.axvline(vlinex,linestyle=':',color='k')
            for hliney in hlinelist:
                fig_ax1.axhline(hliney,linestyle=':',color='k')
            #Add well IDs
            for row in range(baseLayoutDf.shape[0]):
                fig_ax1.annotate(infoDf.values[row][0],(baseLayoutDf.iloc[row,0],baseLayoutDf.iloc[row,1]),ha='center',va='center',size=7)
            #Add plateIDs
            for row in range(baseLayoutDf.shape[0]):
                if infoDf.values[row][1] != 'DoNotLabel':
                    fig_ax1.annotate(infoDf.values[row][1],(baseLayoutDf.iloc[row,0]-0.6,baseLayoutDf.iloc[row,1]+0.5),size=7,ha='center',va='center')
        
        self.canvas = FigureCanvasTkAgg(fig,master=plotFrame)
        modifyPlatePlot()
        self.path = sns.scatterplot(data=baseLayoutDf,x='x',y='y',ax=fig_ax1,color='#808080',s=200,marker='o',alpha=0.5)
        self.canvas.draw()
        self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
        self.trueBackground = self.canvas.copy_from_bbox(fig_ax1.bbox)

        #background = fig.canvas.copy_from_bbox(fig_ax1.bbox)
        #modifyPlatePlot()

        #self.canvas = FigureCanvasTkAgg(fig,master=plotFrame)
        #self.canvas.draw()
        self.canvas.get_tk_widget().pack()

        def line_select_callback(eclick, erelease):
            'eclick and erelease are the press and release events'
            x1, y1 = eclick.xdata, eclick.ydata
            x2, y2 = erelease.xdata, erelease.ydata

        def toggle_selector(event):
            if event.key in ['Q', 'q'] and toggle_selector.RS.active:
                toggle_selector.RS.set_active(False)
            if event.key in ['A', 'a'] and not toggle_selector.RS.active:
                toggle_selector.RS.set_active(True)
         
        rectpropsdict = {'facecolor':'#FF0000','alpha':0.2,'edgecolor':'#FF0000'}
        toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
        self.ts = toggle_selector.RS
        self.canvas.mpl_connect('key_press_event', toggle_selector)
        
        def selectWells(mark):
            wellSelectionBox = toggle_selector.RS.corners
            ll = np.array([wellSelectionBox[0][0], wellSelectionBox[1][0]])  # lower-left
            ur = np.array([wellSelectionBox[0][2], wellSelectionBox[1][2]])  # upper-right
            inidx = np.all(np.logical_and(ll <= self.currentLayout.values[:,:2], self.currentLayout.values[:,:2] <= ur), axis=1)
            inbox = self.currentLayout.loc[inidx]
            changedInbox = inbox.copy()
            if mark:
                changedInbox['key'] = 0
                self.currentLayout.loc[inidx] = changedInbox 
                updatePlatePlot(changedInbox)
            else:
                changedInbox['key'] = -1
                self.currentLayout.loc[inidx] = changedInbox 
                updatePlatePlot(changedInbox)
        
        def updatePlatePlot(newlayout):
            self.path = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,marker='o',color=['#ffffff'])
            #g1 = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,style='key',color=['#ffffff'],markers=['o','X'],style_order=[-1,0])
            self.path = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,style='key',markers=['o','X'],style_order=[-1,0],alpha=0.5,hue_order=[-1,0],hue='key',palette=['#808080','#FF0000'])
            fig_ax1.legend_.remove()
            fig_ax1.draw_artist(self.path)
            self.canvas.blit(fig_ax1.bbox)
            self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
            toggle_selector.RS.background = self.background 
            #self.canvas.draw()
        
        def clearWells():
            MsgBox = tk.messagebox.askquestion ('Warning','Are you sure you want to clear all wells?',icon = 'warning')
            if MsgBox == 'yes':
                self.currentLayout = baseLayoutDf.copy()
                self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,style='key',color=['#ffffff'],markers=['o','X'],style_order=[-1,0])
                self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,marker='o',alpha=0.5,color=['#808080'])
                fig_ax1.legend_.remove()
                self.canvas.restore_region(self.trueBackground)
                self.canvas.blit(fig_ax1.bbox)
                self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
                toggle_selector.RS.background = self.background 
                #self.currentLayout = baseLayoutDf.copy()
                #self.canvas.draw()
        
        def collectInputs():
            master.switch_frame(PlateLayoutPage,folderName,self.currentLayout['key'],levels,levelValues,maxNumLevelValues,numRowPlates,numColumnPlates,plateDimensions,dataTypeList)

        def updateExperimentPlot():
            self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,marker='o',color=['#ffffff'])
            #g1 = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,style='key',color=['#ffffff'],markers=['o','X'],style_order=[-1,0])
            self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,style='key',markers=['o','X'],style_order=[-1,0],alpha=0.5,hue_order=[-1,0],hue='key',palette=['#808080','#FF0000'])
            fig_ax1.legend_.remove()
            fig_ax1.draw_artist(self.path)
            self.canvas.blit(fig_ax1.bbox)
            self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
            toggle_selector.RS.background = self.background 
            #self.canvas.draw()

        def repeatSelection(direction):
            reshapedList = []
            for column in self.currentLayout.columns:
                reshapedList.append(np.matrix(np.reshape(self.currentLayout[column].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates))))
            keyMatrix = reshapedList[2]
            
            currentPlateElementsRow = np.where(keyMatrix != -1)[0]
            currentPlateElementsCol = np.where(keyMatrix != -1)[1]
            plateToRepeat = keyMatrix[np.ix_(np.unique(currentPlateElementsRow),np.unique(currentPlateElementsCol))]
            fullKeyLayout = keyMatrix.copy()
            if direction == 'h':
                repeatedLayout = np.tile(plateToRepeat,numColumnPlates)
                fullKeyLayout[np.ix_(np.unique(currentPlateElementsRow),list(range(plateDimensions[1]*numColumnPlates)))] = repeatedLayout
            else:
                repeatedLayout = np.tile(plateToRepeat,(numRowPlates,1))
                fullKeyLayout[np.ix_(list(range(plateDimensions[0]*numRowPlates)),np.unique(currentPlateElementsCol))] = repeatedLayout
            
            unrolledKeys = fullKeyLayout.flatten()
            self.currentLayout['key'] = unrolledKeys.T
            updateExperimentPlot()

        selectionWindow = tk.Frame(self)
        selectionWindow.grid(row=1,column=0)
        tk.Button(selectionWindow, text="Deselect Wells",command=lambda: selectWells(False)).grid(row=0,column=0)
        tk.Button(selectionWindow, text="Select Wells",command=lambda: selectWells(True)).grid(row=0,column=1)
        
        
        actionWindow = tk.Frame(self)
        actionWindow.grid(row=2,column=0)
        tk.Button(actionWindow, text="Repeat Plate Horizontally",command=lambda: repeatSelection('h')).grid(row=0,column=0)
        tk.Button(actionWindow, text="Repeat Plate Vertically",command=lambda: repeatSelection('v')).grid(row=0,column=1)

        tk.Button(self, text="Clear All Wells",command=lambda: clearWells()).grid(row=3,column=0)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.grid(row=4,column=0)
        
        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs(),font='Helvetica 14 bold').grid(row=0,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(secondaryhomepage,folderName,bPage)).grid(row=0,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).grid(row=0,column=2)

def createLayoutVisual(baseLayoutDf,currentLayout,levelIndex,currentLevel,levelValues,plateDimensions,numRowPlates,numColumnPlates,dt,infoDf,vlinelist,hlinelist):
    
    maxTextLen = len(str(currentLevel))
    for levelVal in levelValues[levelIndex]:
        if len(str(levelVal)) > maxTextLen:
            maxTextLen = len(str(levelVal))
    
    if numRowPlates == 1 and numColumnPlates > colwrap:
        colwrapBool = True
    else:
        colwrapBool = False

    if colwrapBool:
        visualNumRowPlates = math.ceil(numColumnPlates/colwrap)
        visualNumColumnPlates = colwrap
    else:
        visualNumRowPlates = numRowPlates
        visualNumColumnPlates = numColumnPlates 
    fig = plt.figure(figsize=(3*visualNumColumnPlates*(plateDimensions[0]/8)+1+(0.125*maxTextLen), 2.5*visualNumRowPlates*(plateDimensions[1]/12)*figLengthScaling),tight_layout=True)
    #fig_ax1 = fig.add_subplot(111)
    gs = fig.add_gridspec(1, 1)
    fig_ax1 = fig.add_subplot(gs[0])
    
    #Remove all axis elements
    fig_ax1.set_xlim((0, max(baseLayoutDf['x'])+1))
    fig_ax1.set_ylim((0, max(baseLayoutDf['y'])+1))
    fig_ax1.set_xticks([])
    fig_ax1.set_yticks([])
    fig_ax1.set_xlabel('')
    fig_ax1.set_ylabel('')
    #Add plate dividing lines
    for vlinex in vlinelist:
        fig_ax1.axvline(vlinex,linestyle=':',color='k')
    for hliney in hlinelist:
        fig_ax1.axhline(hliney,linestyle=':',color='k')
    #Add well IDs
    for row in range(baseLayoutDf.shape[0]):
        fig_ax1.annotate(infoDf.values[row][0],(baseLayoutDf.iloc[row,0],baseLayoutDf.iloc[row,1]),ha='center',va='center',size=7)
    #Add plateIDs
    for row in range(baseLayoutDf.shape[0]):
        if infoDf.values[row][1] != 'DoNotLabel':
            fig_ax1.annotate(infoDf.values[row][1],(baseLayoutDf.iloc[row,0]-0.6,baseLayoutDf.iloc[row,1]+0.5),size=7,ha='center',va='center')
    
    currentpalette = ['#808080']+sns.color_palette('husl',len(levelValues[levelIndex])).as_hex()
    newLayoutDf = baseLayoutDf.copy()
    newLayoutDf['key'] = currentLayout.flatten()

    g1 = sns.scatterplot(data=baseLayoutDf,x='x',y='y',ax=fig_ax1,color='#ffffff',s=200,marker='o')
    if -1 in list(newLayoutDf['key']) or 'blank' in list(newLayoutDf['key']):
        hueorder = [-1]
        modifiedPalette = ['#808080']
        ogpalette = modifiedPalette.copy()
    else:
        hueorder = []
        modifiedPalette = []
        ogpalette = modifiedPalette.copy()
    for i in range(len(levelValues[levelIndex])):
        if i in list(pd.unique(newLayoutDf['key'])):
            hueorder.append(i)
            modifiedPalette.append(currentpalette[i+1])
    blanksExist = False 
    for i,val in enumerate(newLayoutDf['key']):
        if val == 'blank':
            newLayoutDf['key'].iloc[i] = -1
            newLayoutDf['blank'].iloc[i] = -1
            blanksExist = True
        else:
            newLayoutDf['blank'].iloc[i] = 0 
    g1 = sns.scatterplot(data=newLayoutDf,x='x',y='y',ax=fig_ax1,hue='key',hue_order=hueorder,palette=modifiedPalette,s=200,markers=['X','o'],alpha=0.5,style='blank',style_order=[-1,0])
    titlelabels = [currentLevel+': ']
    titlecolors = ['black']
    
    for i,levelValue in enumerate(levelValues[levelIndex]):
        titlelabels.append(str(levelValue))
        titlecolors.append(modifiedPalette[i+len(ogpalette)])
    #plt.savefig('plateLayout-'+currentLevel+'.png',bbox_inches='tight')
    if 'plateLayouts' not in os.listdir('plots'):
        os.mkdir('plots'+dirSep+'plateLayouts')
    legendHandlesLabels = fig_ax1.get_legend_handles_labels()
    i=0
    newLegendLabels = []
    newLegendHandles = []
    legendHandles = legendHandlesLabels[0]
    currentLevelValues = levelValues[levelIndex]
    if blanksExist:
        offset = 1
    else:
        offset = 0
    for legendHandle in legendHandles:
        #Skips the style legend handles
        if i < len(currentLevelValues)+1:
            if i == 0:
                #modifiedLevel = currentLevel.translate(str.maketrans({"-":  r"\-","]":  r"\]","\\": r"\\","^":  r"\^","$":  r"\$","*":  r"\*",".":  r"\.","_":  r"\_","(":  r"\(",")":  r"\)","[":  r"\[","%": r"\%"}))
                modifiedLevel = currentLevel
                newLegendLabels.append('$\\bf'+modifiedLevel+'$')
                newLegendHandles.append(legendHandle)
            else:
                newLegendLabels.append(currentLevelValues[i-1])
                newLegendHandles.append(legendHandles[i+offset])
        i+=1
    fig_ax1.legend(bbox_to_anchor=(1, 1),frameon=False,handles=newLegendHandles, labels=newLegendLabels)
    plt.savefig('plots'+dirSep+'plateLayouts'+dirSep+'plateLayout-'+currentLevel+'-'+dt+'.png',bbox_inches='tight')
    plt.clf()

class PlateLayoutPage(tk.Frame):
    def __init__(self, master,folderName,blankWells,levels,levelValues,maxNumLevelValues,numRowPlates,numColumnPlates,plateDimensions,dataTypeList):
        
        self.root = master.root
        tk.Frame.__init__(self, master)
        
        tk.Label(self,text='Label Layout Page:',font=('Arial',20)).grid(row=0,column=0)
        
        plotFrame = tk.Frame(self)
        plotFrame.grid(row=9,column=0)
        
        if numRowPlates == 1 and numColumnPlates > colwrap:
            colwrapBool = True
        else:
            colwrapBool = False
        
        if colwrapBool:
            visualNumRowPlates = math.ceil(numColumnPlates/colwrap)
            visualNumColumnPlates = colwrap
        else:
            visualNumRowPlates = numRowPlates
            visualNumColumnPlates = numColumnPlates 
        fig = plt.figure(figsize=(3*visualNumColumnPlates*(plateDimensions[0]/8), 2*visualNumRowPlates*(plateDimensions[1]/12)*figLengthScaling),tight_layout=True)
        #fig = plt.figure(figsize=(3*numColumnPlates*(plateDimensions[0]/8), 2*numRowPlates*(plateDimensions[1]/12)*figLengthScaling),tight_layout=True)
        gs = fig.add_gridspec(1, 1)
        fig_ax1 = fig.add_subplot(gs[0])
        

        #baseLayoutDf,infoDf,vlinelist,hlinelist = returnBaseLayout([16,24],2,2)
        baseLayoutDf,infoDf,vlinelist,hlinelist = returnBaseLayout(plateDimensions,numRowPlates,numColumnPlates)
        baseLayoutDf['blank'] = blankWells
        
        blankMatrix = np.matrix(np.reshape(baseLayoutDf['blank'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates)))
        self.blankRowsToDisregard = []
        self.blankColumnsToDisregard = []
        self.blankPlateRowsToDisregard = []
        self.blankPlateColumnsToDisregard = []
        for row in range(blankMatrix.shape[0]):
            if np.all(blankMatrix[row,:] == 0):
                self.blankRowsToDisregard.append(row)
                if row < plateDimensions[0]:
                    self.blankPlateRowsToDisregard.append(row)
        for col in range(blankMatrix.shape[1]):
            if np.all(blankMatrix[:,col] == 0):
                self.blankColumnsToDisregard.append(col)
                if col < plateDimensions[1]:
                    self.blankPlateColumnsToDisregard.append(col)
        
        self.currentLayout = baseLayoutDf.copy()
        self.infoDf = infoDf.copy()
        self.allLayouts = [baseLayoutDf.copy()]*len(levels)
        self.levelIndex = 0
        self.levelValueIndex = 0
        self.currentpalette = ['#808080']+sns.color_palette('husl',len(levelValues[self.levelIndex])).as_hex()

        def modifyPlatePlot():
            #Remove all axis elements
            fig_ax1.set_xlim((0, max(baseLayoutDf['x'])+1))
            fig_ax1.set_ylim((0, max(baseLayoutDf['y'])+1))
            fig_ax1.set_xticks([])
            fig_ax1.set_yticks([])
            fig_ax1.set_xlabel('')
            fig_ax1.set_ylabel('')
            #Add plate dividing lines
            for vlinex in vlinelist:
                fig_ax1.axvline(vlinex,linestyle=':',color='k')
            for hliney in hlinelist:
                fig_ax1.axhline(hliney,linestyle=':',color='k')
            #Add well IDs
            for row in range(baseLayoutDf.shape[0]):
                fig_ax1.annotate(infoDf.values[row][0],(baseLayoutDf.iloc[row,0],baseLayoutDf.iloc[row,1]),ha='center',va='center',size=7)
            #Add plateIDs
            for row in range(baseLayoutDf.shape[0]):
                if infoDf.values[row][1] != 'DoNotLabel':
                    fig_ax1.annotate(infoDf.values[row][1],(baseLayoutDf.iloc[row,0]-0.6,baseLayoutDf.iloc[row,1]+0.5),size=7,ha='center',va='center')

        self.canvas = FigureCanvasTkAgg(fig,master=plotFrame)
        modifyPlatePlot()
        self.path = sns.scatterplot(data=baseLayoutDf,x='x',y='y',ax=fig_ax1,color='#808080',alpha=0.5,s=200,markers=['o','X'],style='blank',style_order=[-1,0])
        fig_ax1.legend_.remove()
        self.canvas.draw()
        self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
        self.trueBackground = self.canvas.copy_from_bbox(fig_ax1.bbox)
        
        #self.canvas.restore_region(background)
        #fig_ax1.draw_artist(self.path)
        #self.canvas.blit(fig_ax1.bbox)

        self.canvas.get_tk_widget().pack()
        
        def line_select_callback(eclick, erelease):
            'eclick and erelease are the press and release events'
            x1, y1 = eclick.xdata, eclick.ydata
            x2, y2 = erelease.xdata, erelease.ydata

        def toggle_selector(event):
            if event.key in ['Q', 'q'] and toggle_selector.RS.active:
                toggle_selector.RS.set_active(False)
            if event.key in ['A', 'a'] and not toggle_selector.RS.active:
                toggle_selector.RS.set_active(True)
         
        rectpropsdict = {'facecolor':self.currentpalette[self.levelValueIndex+1],'alpha':0.2,'edgecolor':self.currentpalette[self.levelValueIndex+1]}
        toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
        self.ts = toggle_selector.RS
        toggle_selector.RS.background = self.trueBackground 

        def updatePlatePlot(newlayout,key):
            currentcolor = self.currentpalette[key]
            self.path = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,color='#ffffff',marker='o')
            self.path = sns.scatterplot(data=newlayout,x='x',y='y',ax=fig_ax1,s=200,alpha=0.5,markers=['o','X'],style='blank',style_order=[-1,0],color=currentcolor)
            fig_ax1.legend_.remove()
        
        def clearWells():
            MsgBox = tk.messagebox.askquestion ('Warning','Are you sure you want to clear all wells?',icon = 'warning')
            if MsgBox == 'yes':
                self.currentLayout = baseLayoutDf.copy()
                self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,markers=['o','X'],style='blank',style_order=[-1,0],color='#ffffff')
                self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,s=200,markers=['o','X'],alpha=0.5,style='blank',style_order=[-1,0],color='#808080')
                fig_ax1.legend_.remove()
                self.canvas.restore_region(self.trueBackground)
                self.canvas.blit(fig_ax1.bbox)
                self.background = self.trueBackground
                toggle_selector.RS.background = self.trueBackground 
                #self.canvas.draw()

        def changeLevelValue(advance):
            if advance:
                self.levelValueIndex += 1
            else:
                self.levelValueIndex -= 1
            
            if self.levelValueIndex == len(levelValues[self.levelIndex])-1:
                self.nextLevelValueButton['state'] = 'disabled'
            else:
                self.nextLevelValueButton['state'] = 'normal'
            if self.levelValueIndex == 0:
                self.previousLevelValueButton['state'] = 'disabled'
            else:
                self.previousLevelValueButton['state'] = 'normal'
            
            self.levelValueIndex = max([0,self.levelValueIndex])
            self.levelValueIndex = min([len(levelValues[self.levelIndex])-1,self.levelValueIndex])
            
            changeLevelValueInLevelValueLabelList()
            rectpropsdict = {'facecolor':self.currentpalette[self.levelValueIndex+1],'alpha':0.2,'edgecolor':self.currentpalette[self.levelValueIndex+1]}
            toggle_selector.RS.set_active(False)
            toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
            fig_ax1.draw_artist(self.path)
            self.canvas.blit(fig_ax1.bbox)
            self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
            toggle_selector.RS.background = self.background 
            #self.canvas.draw()
        
        def updateExperimentPlot():
            self.path = sns.scatterplot(data=baseLayoutDf,x='x',y='y',ax=fig_ax1,color='#ffffff',s=200,marker='o')
            if -1 in list(self.currentLayout['key']):
                hueorder = [-1]
                modifiedPalette = ['#808080']
            else:
                hueorder = []
                modifiedPalette = []
            for i in range(len(levelValues[self.levelIndex])):
                if i in list(pd.unique(self.currentLayout['key'])):
                    hueorder.append(i)
                    modifiedPalette.append(self.currentpalette[i+1])
            self.path = sns.scatterplot(data=self.currentLayout,x='x',y='y',ax=fig_ax1,hue='key',hue_order=hueorder,palette=modifiedPalette,s=200,markers=['o','X'],alpha=0.5,style='blank',style_order=[-1,0])
            fig_ax1.legend_.remove()
            toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
            fig_ax1.draw_artist(self.path)
            self.canvas.blit(fig_ax1.bbox)
            self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
            toggle_selector.RS.background = self.background 
            #self.canvas.draw()
        
        def changeLevel(advance):
            self.levelValueIndex = 0
            self.allLayouts[self.levelIndex] = self.currentLayout.copy()
            if advance:
                self.levelIndex+=1
            else:
                self.levelIndex-=1
            
            if self.levelIndex == len(levels)-1:
                self.FinishButton['state'] = 'normal'
                self.nextLevelButton['state'] = 'disabled'
            else:
                self.FinishButton['state'] = 'disabled'
                self.nextLevelButton['state'] = 'normal'
            if self.levelIndex == 0:
                self.previousLevelButton['state'] = 'disabled'
            else:
                self.previousLevelButton['state'] = 'normal'
            
            self.nextLevelValueButton['state'] = 'normal'
            self.previousLevelValueButton['state'] = 'disabled'
            
            self.levelIndex = max([0,self.levelIndex])
            self.levelIndex = min([len(levels)-1,self.levelIndex])
            
            self.currentLayout = self.allLayouts[self.levelIndex].copy()
            self.currentpalette = ['#808080']+sns.color_palette('husl',len(levelValues[self.levelIndex])).as_hex()
            
            changeLevelInLevelLabelList()
            changeLevelInLevelValueLabelList()
            #rectpropsdict = {'facecolor':self.currentpalette[self.levelValueIndex+1],'alpha':0.2,'edgecolor':self.currentpalette[self.levelValueIndex+1]}
            #toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
            updateExperimentPlot()

        #Level labels
        levelLabelWindow = tk.Frame(self)
        levelLabelWindow.grid(row=1,column=0)
        
        levelTitle = tk.Label(levelLabelWindow,text='Level: ')
        levelTitle.grid(row=0,column=0,sticky=tk.W)
        levelTitle.configure(font='Helvetica 18 bold')
        self.levelLabelList = []
        for i,level in enumerate(levels):
            currentlabel = tk.Label(levelLabelWindow,text=level)
            if i == self.levelIndex:
                currentlabel.configure(font='Helvetica 18',borderwidth=2, relief="solid")
            else:
                currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
            currentlabel.grid(row=0,column=i+1,sticky=tk.W, padx=2)
            self.levelLabelList.append(currentlabel)
        
        def changeLevelInLevelLabelList():
            for i,level in enumerate(levels):
                currentlabel = self.levelLabelList[i]
                if i == self.levelIndex:
                    currentlabel.configure(font='Helvetica 18',borderwidth=2, relief="solid")
                else:
                    currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
        
        self.previousLevelButton = tk.Button(levelLabelWindow, text="Previous",command=lambda: changeLevel(False))
        self.previousLevelButton.grid(row=0,column=len(levels)+1,sticky=tk.W)
        self.previousLevelButton['state'] = 'disabled'
        self.nextLevelButton = tk.Button(levelLabelWindow, text="Next",command=lambda: changeLevel(True))
        self.nextLevelButton.grid(row=0,column=len(levels)+2,sticky=tk.W)

        #Level value labels
        levelValueLabelWindow = tk.Frame(self)
        levelValueLabelWindow.grid(row=2,column=0)

        levelValueTitle = tk.Label(levelValueLabelWindow,text='Level Value: ')
        levelValueTitle.grid(row=0,column=0,sticky=tk.W+tk.E)
        levelValueTitle.configure(font='Helvetica 18 bold')
        #Need to create maxNumLevelValue labels; only fill in appropriate labels for current level
        self.levelValueLabelList = []
        maxNumLevelValues = len(max(levelValues,key=len))
        if maxNumLevelValues%2 == 0:
            maxNumLevelValues+=1
        labelIndexList = list(range(maxNumLevelValues))
        median = labelIndexList[int(len(labelIndexList)/2)]
        
        self.blanktext = ''

        #Instantiate all level value labels
        numCurrentLevelValues = len(levelValues[self.levelIndex])
        currentLabelIndexList = labelIndexList[median-int(numCurrentLevelValues/2):median-int(numCurrentLevelValues/2)+numCurrentLevelValues]
        j=0
        for i in range(maxNumLevelValues):
            if i in currentLabelIndexList:
                levelValue = levelValues[self.levelIndex][j]
                currentlabel = tk.Label(levelValueLabelWindow,text=levelValue,fg = self.currentpalette[j+1])
                if j == self.levelValueIndex:
                    currentlabel.configure(font='Helvetica 18',borderwidth=2, relief="solid")
                else:
                    currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
                j+=1
            else:
                currentlabel = tk.Label(levelValueLabelWindow,text=self.blanktext)
            currentlabel.grid(row=0,column=i+1,sticky=tk.W+tk.E,padx=2)
            self.levelValueLabelList.append(currentlabel)
        
        def changeLevelValueInLevelValueLabelList():
            numCurrentLevelValues = len(levelValues[self.levelIndex])
            currentLabelIndexList = labelIndexList[median-int(numCurrentLevelValues/2):median-int(numCurrentLevelValues/2)+numCurrentLevelValues]
            j=0
            for i in range(maxNumLevelValues):
                currentlabel = self.levelValueLabelList[i]
                if i in currentLabelIndexList:
                    if currentlabel['text'] != self.blanktext:
                        if j == self.levelValueIndex:
                            currentlabel.configure(font='Helvetica 18',borderwidth=2, relief="solid")
                        else:
                            currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
                    else:
                        currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
                    j+=1

        self.previousLevelValueButton = tk.Button(levelValueLabelWindow, text="Previous",command=lambda: changeLevelValue(False))
        self.previousLevelValueButton.grid(row=0,column=maxNumLevelValues+1,sticky=tk.W)
        self.previousLevelValueButton['state'] = 'disabled'
        self.nextLevelValueButton = tk.Button(levelValueLabelWindow, text="Next",command=lambda: changeLevelValue(True))
        self.nextLevelValueButton.grid(row=0,column=maxNumLevelValues+2,sticky=tk.W)
        
        def changeLevelInLevelValueLabelList():
            numCurrentLevelValues = len(levelValues[self.levelIndex])
            currentLabelIndexList = labelIndexList[median-int(numCurrentLevelValues/2):median-int(numCurrentLevelValues/2)+numCurrentLevelValues]
            j = 0
            for i in range(maxNumLevelValues):
                currentlabel = self.levelValueLabelList[i]
                if i in currentLabelIndexList:
                    levelValue = levelValues[self.levelIndex][j]
                    currentlabel.configure(text=levelValue,fg = self.currentpalette[j+1])
                    if j == self.levelValueIndex:
                        currentlabel.configure(font='Helvetica 18',borderwidth=2, relief="solid")
                    else:
                        currentlabel.configure(font='Helvetica 18',borderwidth=0, relief="solid")
                    j+=1
                else:
                    currentlabel.configure(text=self.blanktext,borderwidth=0, relief="solid")
            if numColumnPlates <= 2:
                currentlabel.update()
        
        def selectWells(mark,immediate):

            wellSelectionBox = toggle_selector.RS.corners
            ll = np.array([wellSelectionBox[0][0], wellSelectionBox[1][0]])  # lower-left
            ur = np.array([wellSelectionBox[0][2], wellSelectionBox[1][2]])  # upper-right
            inidx = np.all(np.logical_and(ll <= self.currentLayout.values[:,:2], self.currentLayout.values[:,:2] <= ur), axis=1)
            inbox = self.currentLayout.loc[inidx]
            changedInbox = inbox.copy().loc[inbox.blank != 0,:]
            #Remove blank wells from selection to change
            inidx2 = inidx.copy()
            for row in range(self.currentLayout.shape[0]):
                if self.currentLayout.iloc[row,:]['blank'] == 0:
                    inidx2[row] = False

            if mark:
                changedInbox.loc[:,'key'] = self.levelValueIndex
                self.currentLayout.loc[inidx2] = changedInbox
                updatePlatePlot(changedInbox,self.levelValueIndex+1)
                if immediate == 'yes' and self.levelValueIndex != maxNumLevelValues-1:
                    changeLevelValue(True) 
                else:
                    rectpropsdict = {'facecolor':self.currentpalette[self.levelValueIndex+1],'alpha':0.2,'edgecolor':self.currentpalette[self.levelValueIndex+1]}
                    toggle_selector.RS = RectangleSelector(fig_ax1, line_select_callback, useblit=True,button=[1, 3], minspanx=1, minspany=1,spancoords='pixels',interactive=True,props=rectpropsdict)
                    fig_ax1.draw_artist(self.path)
                    self.canvas.blit(fig_ax1.bbox)
                    self.background = self.canvas.copy_from_bbox(fig_ax1.bbox)
                    toggle_selector.RS.background = self.background 
            else:
                changedInbox['key'] = -1
                self.currentLayout.loc[inidx] = changedInbox 
                updatePlatePlot(changedInbox,0)
                toggle_selector.RS.background = self.background 
                fig_ax1.draw_artist(self.path)
                self.canvas.blit(fig_ax1.bbox)

        def copySelection():
            wellSelectionBox = toggle_selector.RS.corners
            ll = np.array([wellSelectionBox[0][0], wellSelectionBox[1][0]])  # lower-left
            ur = np.array([wellSelectionBox[0][2], wellSelectionBox[1][2]])  # upper-right
            inidx = np.all(np.logical_and(ll <= self.currentLayout.values[:,:2], self.currentLayout.values[:,:2] <= ur), axis=1)
            inbox = self.currentLayout.loc[inidx]
            changedInbox = inbox.copy().loc[inbox.blank != 0,:]
            #Remove blank wells from selection to change
            self.storedSelection = changedInbox['key'].values
        
        def pasteSelection():
            wellSelectionBox = toggle_selector.RS.corners
            ll = np.array([wellSelectionBox[0][0], wellSelectionBox[1][0]])  # lower-left
            ur = np.array([wellSelectionBox[0][2], wellSelectionBox[1][2]])  # upper-right
            inidx = np.all(np.logical_and(ll <= self.currentLayout.values[:,:2], self.currentLayout.values[:,:2] <= ur), axis=1)
            inbox = self.currentLayout.loc[inidx]
            changedInbox = inbox.copy().loc[inbox.blank != 0,:]
            #Remove blank wells from selection to change
            inidx2 = inidx.copy()
            for row in range(self.currentLayout.shape[0]):
                if self.currentLayout.iloc[row,:]['blank'] == 0:
                    inidx2[row] = False
            changedInbox.loc[:,'key'] = self.storedSelection
            self.currentLayout.loc[inidx2] = changedInbox
            updateExperimentPlot()

        def operateOnSelection(action,area,direction):
            
            keyMatrix = np.matrix(np.reshape(self.currentLayout['key'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates)))
            selectedElements = np.where(keyMatrix != -1)
            selectionToOperateOn = keyMatrix[np.ix_(np.unique(selectedElements[0]),np.unique(selectedElements[1]))]
            offset = [1,1]
            if area == 'experiment':
                areashape = keyMatrix.shape
                blankRows = self.blankRowsToDisregard
                blankColumns = self.blankColumnsToDisregard
            else:
                areashape = (plateDimensions[0],plateDimensions[1])
                blankRows = self.blankPlateRowsToDisregard
                blankColumns = self.blankPlateColumnsToDisregard
            blankArray = [blankRows,blankColumns]
            tiledLevelLayout = np.ones(keyMatrix.shape)*-1
            tilingMatrixList = []
            if action == 'tile':
                if direction == 'h':
                    numTiles = math.ceil(len(levelValues[self.levelIndex])/(areashape[1]-len(blankColumns)))
                    spillover = (areashape[1]-len(blankColumns))%len(levelValues[self.levelIndex])
                    shapeIndex = 1 
                else:
                    numTiles = math.ceil(len(levelValues[self.levelIndex])/(areashape[0]-len(blankRows)))
                    spillover = (areashape[0]-len(blankRows))%len(levelValues[self.levelIndex])
                    shapeIndex = 0
                if spillover == 0:
                    for tileIndex in range(len(levelValues[self.levelIndex])):
                        tilingMatrix = np.ones(selectionToOperateOn.shape) * tileIndex
                        tilingMatrixList.append(tilingMatrix)
                    if direction == 'h':
                        fullTilingMatrix = np.hstack(tilingMatrixList)
                    else:
                        fullTilingMatrix = np.vstack(tilingMatrixList)
                else:
                    splitLevelValues = np.array_split(np.array(levelValues[self.levelIndex]),int(numTiles))
                    splitLevelValues2 = np.array_split(levelValues[self.levelIndex],numTiles+1)
                    fullTilingMatrixList = []
                    for i,splitLevelValue in enumerate(splitLevelValues):
                        partialTilingMatrixList = []
                        for tileIndex in range(len(splitLevelValue)):
                            tilingMatrix = np.ones(selectionToOperateOn.shape) * (tileIndex + i*len(splitLevelValues[0]))
                            partialTilingMatrixList.append(tilingMatrix)
                        if len(levelValues[self.levelIndex])%(areashape[shapeIndex]-len(blankArray[shapeIndex])) != 0:
                            for spilloverValue in range(int(spillover/selectionToOperateOn.shape[shapeIndex])):
                                tilingMatrix = np.ones(selectionToOperateOn.shape) * -1
                                partialTilingMatrixList.append(tilingMatrix)
                        if direction == 'h':
                            partialTilingMatrix = np.hstack(partialTilingMatrixList)
                        else:
                            partialTilingMatrix = np.vstack(partialTilingMatrixList)
                        fullTilingMatrixList.append(partialTilingMatrix)
                    fullTilingMatrix = np.ones(areashape)*-1
                    if direction == 'h':
                        fullTilingMatrix[:int(numTiles),:int((areashape[1]-len(blankColumns)))] = np.vstack(fullTilingMatrixList)
                    else:
                        fullTilingMatrix[:int((areashape[0]-len(blankRows))),:int(numTiles)] = np.hstack(fullTilingMatrixList)
            else:
                if direction == 'h':
                    numRepeats = int((areashape[1]-len(self.blankColumnsToDisregard))/selectionToOperateOn.shape[1])
                    fullTilingMatrix = np.tile(selectionToOperateOn,numRepeats)
                else:
                    numRepeats = int((areashape[0]-len(self.blankRowsToDisregard))/selectionToOperateOn.shape[0])
                    fullTilingMatrix = np.tile(selectionToOperateOn,(numRepeats,1))
            tilerow = 0
            if area == 'plate':
                if direction == 'h':
                    shapeIndex = 0
                else:
                    shapeIndex = 1
                #Not sure what this does
                if fullTilingMatrix.shape[shapeIndex] != (areashape[shapeIndex]-len(blankArray[shapeIndex])):
                    off = math.ceil((areashape[shapeIndex]-len(blankArray[shapeIndex]))/fullTilingMatrix.shape[shapeIndex])
                    offset[shapeIndex] = off
            if action == 'repeat':
                for row in range(int(areashape[0]/offset[0])):
                    tilecol = 0
                    for col in range(int(areashape[1]/offset[1])):
                        if row not in blankArray[0]:
                            if col not in blankArray[1]:
                                tiledLevelLayout[row,col] = fullTilingMatrix[tilerow,tilecol]
                                tilecol += 1
                    if row not in blankArray[0]:
                        tilerow += 1
            else:
                for row in range(areashape[0]):
                    tilecol = 0
                    for col in range(areashape[1]):
                        if row not in blankArray[0]:
                            if col not in blankArray[1]:
                                tiledLevelLayout[row,col] = fullTilingMatrix[tilerow,tilecol]
                                tilecol += 1
                                if tilecol >= fullTilingMatrix.shape[1]:
                                    break
                    if row not in blankArray[0]:
                        tilerow += 1
                        if tilerow >= fullTilingMatrix.shape[0]:
                            break
            fullTiledLevelLayout = tiledLevelLayout.copy()
            """
            for row in range(int(areashape[0]/offset[0])):
                tilecol = 0
                for col in range(int(areashape[1]/offset[1])):
                    if row not in blankArray[0]:
                        print('wat2')
                        if col not in blankArray[1]:
                            print('wat3')
                            tiledLevelLayout[row,col] = fullTilingMatrix[tilerow,tilecol]
                            tilecol += 1
                if row not in blankArray[0]:
                    tilerow += 1
            fullTiledLevelLayout = tiledLevelLayout.copy()
            """
            unrolledKeys = fullTiledLevelLayout.flatten()
            if action == 'tile':
                self.currentLayout['key'] = unrolledKeys
            else:
                self.currentLayout['key'] = unrolledKeys.T
            updateExperimentPlot()

        def collectInputs():
            if self.levelIndex == len(levels)-1:
                self.allLayouts[self.levelIndex] = self.currentLayout.copy()
                finalLayoutDict = {}
                levelValueDict = {}
                keyDict = {}
                for levelKey,layout in enumerate(self.allLayouts):
                    keyMatrix = np.reshape(layout['key'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates))
                    levelValueMatrix = np.empty(keyMatrix.shape, dtype=object)
                    for row in range(keyMatrix.shape[0]):
                        for col in range(keyMatrix.shape[1]):
                            levelValueKey = int(keyMatrix[row][col])
                            if levelValueKey == -1:
                                levelValueMatrix[row,col] = 'blank'
                            else:
                                levelValueMatrix[row,col] = levelValueKey

                    levelValueDict[levelKey] = levelValueMatrix

                wellIDMatrix = np.reshape(self.infoDf['wellID'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates))
                plateIDMatrix = np.reshape(self.infoDf['plateName'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates))
                
                blankMatrix = np.reshape(self.allLayouts[0]['blank'].values,(plateDimensions[0]*numRowPlates,plateDimensions[1]*numColumnPlates))

                finalLayoutDict['plateID'] = plateIDMatrix
                finalLayoutDict['wellID'] = wellIDMatrix
                finalLayoutDict['blank'] = blankMatrix
                finalLayoutDict['keys'] = levelValueDict
                
                keyList = []
                for key in finalLayoutDict['keys']:
                    keyList.append(finalLayoutDict['keys'][key])

                fullLayout = np.dstack(keyList)

                uniqueKeys = []
                nonUniquePositions = []
                for row in range(fullLayout.shape[0]):
                    for col in range(fullLayout.shape[1]):
                        wellKeys = fullLayout[row,col,:].tolist()
                        if 'blank' not in wellKeys:
                            if wellKeys not in uniqueKeys:
                                uniqueKeys.append(wellKeys)
                            else:
                                plateID = finalLayoutDict['plateID'][row,col]
                                wellID = finalLayoutDict['wellID'][row,col]
                                nonUniquePositions.append(plateID+'/'+wellID)
                
                if len(nonUniquePositions) == 0:
                    for dt in dataTypeList:
                        with open('misc'+dirSep+'layoutDict-'+folderName+'-'+dt+'.pkl','wb') as f:
                            pickle.dump(finalLayoutDict,f)
                        for i in finalLayoutDict['keys']:
                            level = levels[i]
                            createLayoutVisual(baseLayoutDf,finalLayoutDict['keys'][i],i,level,levelValues,plateDimensions,numRowPlates,numColumnPlates,','.join(dataTypeList),infoDf,vlinelist,hlinelist)
                    master.switch_frame(backPage,folderName)
                else:
                    tk.messagebox.showinfo("Duplicate well labels", "These wells (plateID/wellID) have duplicate labels. Please correct them and try again:\n\n"+', '.join(nonUniquePositions))
            else:
                print('Not all levels arranged')
        
        selectionWindow = tk.Frame(self)
        selectionWindow.grid(row=3,column=0)
        advanceVar = tk.StringVar()
        advanceVar.set('yes')
        tk.Button(selectionWindow, text="Deselect Wells",command=lambda: selectWells(False,advanceVar.get())).grid(row=0,column=0)
        tk.Button(selectionWindow, text="Select Wells",command=lambda: selectWells(True,advanceVar.get())).grid(row=0,column=1)
        
        tk.Checkbutton(self,text="Immediately advance level value",variable=advanceVar,onvalue='yes',offvalue='no').grid(row=4,column=0)
        
        editWindow = tk.Frame(self)
        editWindow.grid(row=5,column=0)
        tk.Button(editWindow, text="Copy selection",command=lambda: copySelection()).grid(row=0,column=0)
        tk.Button(editWindow, text="Paste selection",command=lambda: pasteSelection()).grid(row=0,column=1)
        
        actionWindow = tk.Frame(self)
        actionWindow.grid(row=6,column=0)

        actionVar = tk.StringVar()
        actionVar.set('tile')
        tileButton = tk.Radiobutton(actionWindow,text="Tile", variable=actionVar, value='tile')
        repeatButton = tk.Radiobutton(actionWindow,text="Repeat", variable=actionVar, value='repeat')
        tileButton.grid(row=0,column=0,sticky=tk.W)
        repeatButton.grid(row=1,column=0,sticky=tk.W)

        areaVar = tk.StringVar()
        areaVar.set('experiment')
        expButton = tk.Radiobutton(actionWindow,text="Experiment", variable=areaVar, value='experiment')
        plateButton = tk.Radiobutton(actionWindow,text="Plate", variable=areaVar, value='plate')
        expButton.grid(row=0,column=1,sticky=tk.W)
        plateButton.grid(row=1,column=1,sticky=tk.W)
        
        directionVar = tk.StringVar()
        directionVar.set('v')
        vButton = tk.Radiobutton(actionWindow,text="Vertically", variable=directionVar, value='v')
        hButton = tk.Radiobutton(actionWindow,text="Horizontally", variable=directionVar, value='h')
        vButton.grid(row=0,column=2,sticky=tk.W)
        hButton.grid(row=1,column=2,sticky=tk.W)

        tk.Button(actionWindow, text="Perform Action",command=lambda: operateOnSelection(actionVar.get(),areaVar.get(),directionVar.get())).grid(row=0,column=3,rowspan=2)

        tk.Button(self, text="Clear All Wells",command=lambda: clearWells()).grid(row=7,column=0)
        
        buttonWindow = tk.Frame(self)
        buttonWindow.grid(row=8,column=0)
        
        self.FinishButton = tk.Button(buttonWindow, text="Finish",command=lambda: collectInputs(),font='Helvetica 14 bold')
        self.FinishButton.grid(row=0,column=0)
        self.FinishButton['state'] = 'disabled'
        #(self, master,folderName,levels,levelValues,maxNumLevelValues,numRowPlates,numColumnPlates,plateDimensions,dataType,shp)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(BlankSelectionPage,folderName,levels,levelValues,maxNumLevelValues,numColumnPlates,plateDimensions,dataTypeList,secondaryhomepage,backPage)).grid(row=0,column=1)
        tk.Button(buttonWindow, text="Quit",command=lambda: quit()).grid(row=0,column=2)



#!/usr/bin/env python3 
import string,os
import pandas as pd
import numpy as np
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'
from plateypus.dataprocessing.miscFunctions import reorderDfByInputOrder

def generateBulkKillingStatistics(experimentParameters,folderName,experimentNumber,dataType,layoutDict,unwrappingOrder='col'):
        numRowPlates = 1 
        numColumnPlates = experimentParameters['numPlates']

        #Combine plate and well IDs into a single ID val for every single sample in the experiment
        identificationMatrix = np.empty(layoutDict['plateID'].shape,dtype=object)
        for row in range(identificationMatrix.shape[0]):
            for col in range(identificationMatrix.shape[1]):
                wellID = layoutDict['wellID'][row,col]
                plateID = layoutDict['plateID'][row,col]
                fullID = plateID+'-'+wellID
                identificationMatrix[row,col] = fullID

        plateNames = np.unique(layoutDict['plateID'])
        
        levelLabelDict = experimentParameters['levelLabelDict']
        if 'Time' in levelLabelDict:
            del levelLabelDict['Time']

        plateLayoutDict = layoutDict['keys']
        blankLayout = layoutDict['blank'].astype(object)
        
        fullExperimentDf,plateLayoutDf = createIncucyteDataframe(plateNames,{**plateLayoutDict,**{'Blank':blankLayout}},{**levelLabelDict,**{'Blank':['Yes','No']}},unwrappingOrder) 
        
        fullExperimentDf = reorderDfByInputOrder(experimentParameters,fullExperimentDf)
        return fullExperimentDf

def createIncucyteDataframe(plateNames,plateLayoutDict,levelLabelDict,unwrappingOrder='col'):
    path = 'inputData'+dirSep+'bulkCSVFiles'+dirSep
    if 'A1_killing.csv' in os.listdir(path):
        csvBool = True
    else:
        csvBool = False
    #Create incucyte data matrix
    dfList = []
    for plateName in plateNames:
        if csvBool:
            plateDf = pd.read_csv(path+plateName+'_killing.csv',header=1).iloc[:,2:]
        else:
            plateDf = pd.read_excel(path+plateName+'_killing.xlsx',header=1).iloc[:,2:]
        dfList.append(plateDf.dropna())
    
    inputDf = pd.concat(dfList,axis=1,keys=string.ascii_uppercase[:len(dfList)]).dropna()
    if csvBool:
        inputDf.index = pd.read_csv(path+plateNames[0]+'_killing.csv',header=1).iloc[:,1].dropna().values
    else:
        inputDf.index = pd.read_excel(path+plateNames[0]+'_killing.xlsx',header=1).iloc[:,1].dropna().values
    
    inputDf.index.name = 'Time'
    
    unwrappingOrderDict = {'col':'F','row':'C'}
    flatteningOrder = unwrappingOrderDict[unwrappingOrder]

    inputPlateLayout = {}
    for levelKey,level in zip(plateLayoutDict,levelLabelDict):
      inputPlateLayout[level] = plateLayoutDict[levelKey]
    
    # Reshape lists so that they are in row/column major format
    reshapedLevelValues = []
    for level in inputPlateLayout:
        plateLayout = inputPlateLayout[level]
        levelValues = levelLabelDict[level]
        for row in range(plateLayout.shape[0]):
            for col in range(plateLayout.shape[1]):
                if level != 'Blank':
                    if plateLayout[row,col] == 'blank':
                        plateLayout[row,col] = 'blank'
                    else:
                        plateLayout[row,col] = levelValues[plateLayout[row,col]]
                else:
                    if plateLayout[row,col] == 0:
                        plateLayout[row,col] = 'blank'
                    else:
                        plateLayout[row,col] = levelValues[plateLayout[row,col]]
        newLayout = [x for x in plateLayout.flatten(order=flatteningOrder) if x != 'blank']
        reshapedLevelValues.append(newLayout)

    #Create multiIndex
    multiIndex = pd.MultiIndex.from_arrays(reshapedLevelValues,names=inputPlateLayout.keys())
    #Drop unused wells (in block plates) from multiIndex
    trueMultiIndex = pd.MultiIndex.from_tuples([x for x in multiIndex.values.tolist() if '' not in x],names=inputPlateLayout.keys())

    #Create plate layout dataframe
    plateLayoutDf = pd.DataFrame(trueMultiIndex.values.tolist(),index=inputDf.columns,columns=list(inputPlateLayout.keys()))
    plateLayoutDf.index.names = ['Plate','Well']

    #Create final dataframe, drop blanks, and swap columns/rows
    if 'Blank' in list(inputPlateLayout.keys()):
        if 'Yes' in inputPlateLayout['Blank'].flatten():
            incucyteDf = pd.DataFrame(inputDf.values,index=inputDf.index,columns=trueMultiIndex).drop('Yes',level='Blank',axis=1).droplevel('Blank',axis=1).replace('na',np.nan)#.astype(float).T
        else:
            incucyteDf = pd.DataFrame(inputDf.values,index=inputDf.index,columns=trueMultiIndex).droplevel('Blank',axis=1).replace('na',np.nan)#.astype(float).T
    else:
        incucyteDf = pd.DataFrame(inputDf.values,index=inputDf.index,columns=trueMultiIndex).replace('na',np.nan)#.astype(float).T
    
    #Correct missing values to zero
    for i in range(incucyteDf.shape[0]):
        for j in range(incucyteDf.shape[1]):
            if type(incucyteDf.iloc[i,j]) == str:
                incucyteDf.iloc[i,j] = 0
    incucyteDf = incucyteDf.astype(float).T

    return incucyteDf,plateLayoutDf



#!/usr/bin/env python3  
import os,pickle,shutil
import statistics
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
import seaborn as sns
#from logicle import logicle,quantile 
from datetime import datetime
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg
import tkinter as tk
import matplotlib.lines as linesmpl
from plateypus.dataprocessing.miscFunctions import find_nearest,returnTicks,returnGates,returnGatesLinear
from scipy.signal import find_peaks
from scipy.stats import gaussian_kde
import os
if os.name == 'nt':
    dirSep = '\\'
else:
    dirSep = '/'

class ProliferationSelectionPage(tk.Frame):
    def __init__(self,master,fName,expn,exdata,shp,bp):
        tk.Frame.__init__(self, master)
        
        global secondaryhomepage
        secondaryhomepage = shp
        global folderName
        folderName = fName
        global expNum
        expNum = expn
        global ex_data
        ex_data = exdata
        global bPage
        bPage = bp
        
        singleCellDf = pd.read_hdf('outputData'+dirSep+'pickleFiles'+dirSep+'initialSingleCellDf-channel-'+folderName+'.h5',key='df')
        markers = list(singleCellDf.columns)
        celltypes = singleCellDf.index.unique('CellType').tolist()
        
        mainWindow = tk.Frame(self)
        mainWindow.pack()

        tk.Label(mainWindow,text='Proliferating:').grid(row=0,column=0,columnspan=2,sticky=tk.W)
        tk.Label(mainWindow,text='Celltypes:').grid(row=1,column=0,sticky=tk.W)
        rbList,rbVarList = [],[]
        for i,celltype in enumerate(celltypes):
            rbVar = tk.BooleanVar(value=False)
            rb = tk.Checkbutton(mainWindow,text=celltype,variable=rbVar)
            rb.grid(row=i+2,column=0,sticky=tk.W)
            rbList.append(rb)
            rbVarList.append(rbVar)
        
        tk.Label(mainWindow,text='Markers:').grid(row=1,column=1,sticky=tk.W)
        rbList2,rbVarList2 = [],[]
        for i,marker in enumerate(markers):
            rbVar = tk.BooleanVar(value=False)
            rb = tk.Checkbutton(mainWindow,text=marker,variable=rbVar)
            rb.grid(row=i+2,column=1,sticky=tk.W)
            rbList2.append(rb)
            rbVarList2.append(rbVar)

        def collectInputs():
            prolifCellTypes = [celltypes[i] for i,x in enumerate(rbVarList) if x.get()]
            prolifMarkers = [markers[i] for i,x in enumerate(rbVarList2) if x.get()]
            #Only select first prolif marker and first prolif celltype for now. TODO: allow multiple proliferation markers/celltypes
            global logicleDataStacked 
            logicleDataStacked = singleCellDf[prolifMarkers[0]].xs((prolifCellTypes[0]),level=('CellType'))
            master.switch_frame(GatingPage)
         

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(secondaryhomepage,folderName,expNum,ex_data,bPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class GatingPage(tk.Frame):
    def __init__(self,master):
        tk.Frame.__init__(self, master)
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()

        mainWindow = tk.Frame(self)
        mainWindow.pack(side=tk.TOP,padx=10,pady=10)
        v1 = tk.StringVar(value='Condition-Time')
        l1 = tk.Label(mainWindow,text = 'Do you want to group all samples, all timepoints, or all conditions together for initial CTV Gating?').grid(row=0,column=0,sticky=tk.W)
        rb1 = tk.Radiobutton(mainWindow,text='Group all Samples',variable=v1,value='Condition-Time')
        rb2 = tk.Radiobutton(mainWindow,text='Group all conditions',variable=v1,value='Condition')
        rb3 = tk.Radiobutton(mainWindow,text='Group all timepoints',variable=v1,value='Time')
        rb1.grid(row=1,column=0,sticky=tk.W)
        rb2.grid(row=2,column=0,sticky=tk.W)
        rb3.grid(row=3,column=0,sticky=tk.W)
        
        tk.Label(mainWindow,text='Maximum number of generations').grid(row=4,column=0)
        e1 = tk.Entry(mainWindow)
        e1.insert(tk.END,'6')
        e1.grid(row=5,column=0)

        #logicleDataStacked = pickle.load(open('outputData/pickleFiles/logicleProliferationDf.pkl','rb'))
        groupedGateList = []

        rowAsAllLevelNames = logicleDataStacked.index.names[:-1]
        rowAsTimePointLevelNames = logicleDataStacked.index.names[-2]
        rowAsConditionLevelNames = logicleDataStacked.index.names[:-2] 
        conditionTuple = []
        for tpl in logicleDataStacked.groupby(level=rowAsConditionLevelNames,sort=False):
            conditionTuple.append(tpl[0])
        timepointTuple = []
        for tpl in logicleDataStacked.groupby(level=rowAsTimePointLevelNames,sort=False):
            timepointTuple.append(tpl[0])
        
        #os.chdir('../../experiments/'+folderName) 
        def collectInputs():
            global groupVariable
            global titleVariable
            global outerVariableValues
            global innerVariableValues
            global titleVariable
            global levelNames
            global numGens
            groupVariable = v1.get()
            numGens = int(e1.get())
            #Group all samples together (no iteration at all)
            if groupVariable == 'Condition-Time':
                levelNames = rowAsAllLevelNames
                outerVariableValues = conditionTuple
                innerVariableValues = timepointTuple
                titleVariable = 'All'
                iterationRange = 1
            else:
                #Group conditions together (iterate through each timepoint):
                if groupVariable == 'Condition':
                    levelNames = rowAsTimePointLevelNames
                    outerVariableValues = timepointTuple
                    innerVariableValues = conditionTuple
                    titleVariable = 'Time'
                #Group timepoints together (iterate through each condition):
                else:
                    levelNames = rowAsConditionLevelNames
                    outerVariableValues = conditionTuple
                    innerVariableValues = timepointTuple
                    titleVariable = 'Condition'
                global logicleDataUnstacked
                logicleDataUnstacked = logicleDataStacked.groupby(level=levelNames,sort=False).first().to_frame('MFI')
                iterationRange = logicleDataUnstacked.shape[0] 
            with open('misc'+dirSep+'iterationRange.pkl','wb') as f:
                pickle.dump(iterationRange,f)
            with open('misc'+dirSep+'rowValIterationRange.pkl','wb') as f:
                pickle.dump(0,f)
            groupedGateList = []
            with open('misc'+dirSep+'groupedGateList.pkl','wb') as f:
                pickle.dump(groupedGateList,f)
            master.switch_frame(Unsplit_Proliferation_Gating_GUI)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Back",command=lambda: master.switch_frame(ProliferationSelectionPage,folderName,expNum,ex_data,secondaryhomepage,bPage)).grid(row=5,column=1)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

class Unsplit_Proliferation_Gating_GUI(tk.Frame):
    def __init__(self,master):
        self.root = master.root
        tk.Frame.__init__(self, master)
        
        experimentNameWindow = tk.Frame(self)
        experimentNameWindow.pack(side=tk.TOP,padx=10,pady=10)
        experimentNameLabel = tk.Label(experimentNameWindow,text=folderName+':').pack()
        
        plotFrame = tk.Frame(self)
        plotFrame.pack()
        row = pickle.load(open('misc'+dirSep+'rowValIterationRange.pkl','rb'))
        groupedGateList = pickle.load(open('misc'+dirSep+'groupedGateList.pkl','rb'))
        if groupVariable == 'Condition-Time':
            currentDf = logicleDataStacked.to_frame('MFI')
        else:
            currentLevelValues = logicleDataUnstacked.iloc[row,:].name
            currentDf = logicleDataStacked.xs(currentLevelValues,level=levelNames).to_frame('MFI')
        plottingDf = currentDf.reset_index()
        if groupVariable == 'Condition-Time' or groupVariable == 'Condition':
            nonEventLevelNames = currentDf.index.names[:-1]
            groupingColumn = []
            for conditionGroupbyTuple in currentDf.groupby(level=nonEventLevelNames,sort=False):
                conditionGroupbyTupleNew = [str(i) for i in conditionGroupbyTuple[0]]
                conditionName = '-'.join(conditionGroupbyTupleNew)
                for event in range(conditionGroupbyTuple[1].shape[0]):
                    groupingColumn.append(conditionName)
            plottingDf[groupVariable] = groupingColumn
        
        #Plot facetgrid of histograms of ctv values of all times for current condition
        kind='kde'
        if groupVariable == 'Condition-Time':
            g = sns.displot(data=plottingDf,x='MFI',color='purple',height=6,kind=kind)
        elif groupVariable == 'Time':
            g = sns.displot(data=plottingDf,x='MFI',hue='Time',height=6,palette=sns.color_palette("Purples", len(pd.unique(plottingDf[groupVariable]))),kind=kind)
        else:
            g = sns.displot(data=plottingDf,x='MFI',hue='Condition',height=6,palette=sns.color_palette("Purples", len(pd.unique(plottingDf['Condition']))),kind=kind)
        #g = sns.FacetGrid(plottingDf,legend_out=True,hue=groupVariable,height=6,palette=sns.color_palette("Purples", len(pd.unique(plottingDf[groupVariable]))))
        #g.map(sns.distplot,'MFI',kde=False,bins=256)
        #g.map(sns.kdeplot,'MFI',bw=15)
        plt.subplots_adjust(top=0.95)
        xtickValues,xtickLabels = returnTicks([-1000,100,1000,10000,100000])
        axis = g.fig.get_axes()[0]
        axis.set_xticks(xtickValues)
        axis.set_xticklabels(xtickLabels)
        
        #startingGateLogicle = 725
        space = np.linspace(0,1024,num=100)
        kde = gaussian_kde(currentDf['MFI'].values)
        kdeVals = kde.evaluate(space)
        maxInd = np.argmax(kdeVals)
        minMaxScaledKDE = kdeVals.reshape(-1,1)
        minMaxScaledKDE = (minMaxScaledKDE - minMaxScaledKDE.min()) / (minMaxScaledKDE.max() - minMaxScaledKDE.min())
        #tempDf = pd.DataFrame({sizeDimension:space,'Density':minMaxScaledKDE[:,0]})
        startingGateLogicle = space[np.argmax(minMaxScaledKDE[:,0] > 0.01)]
        endingGateLogicle = max(space)

        logicleGates = [startingGateLogicle]+[space[x] for x in list(find_peaks(kdeVals*-1)[0]) if space[x] > startingGateLogicle]+[endingGateLogicle]
        logicleGates = logicleGates[::-1]
        #logicleGates = returnGatesLinear(currentDf,startingGateLogicle,numGens)
        maxY = list(plt.gca().get_yticks())[-1]
        X = endingGateLogicle
        Ymin = -1
        Ymax = maxY
        gates = logicleGates[1:]
        
        #sys.exit(0)
        numGates = len(logicleGates)-1
        self.canvas = FigureCanvasTkAgg(g.fig,master=plotFrame)
        self.ax = plt.gca() 
        self.canvas.draw()
        self.canvas.get_tk_widget().pack()
        self.X = X
        self.gates= gates
        self.gateVals = gates
        gateLengths = [y-x for x, y in zip(gates[:-1], gates[1:])][1:]
        self.gateLengths = gateLengths 
        self.numGates = numGates
        x = [X, X]
        y = [Ymin, Ymax]
        
        self.line = linesmpl.Line2D(x, y, picker=5,linestyle=':',color='r')
        self.ax.add_line(self.line)
        self.childlines = []
        for gate in range(self.numGates):
            newX = self.gates[gate]
            childline = linesmpl.Line2D([newX,newX], y,linestyle=':',color='k',picker=5)
            self.childlines.append(childline)
        for childline in self.childlines:
            self.ax.add_line(childline)
        self.canvas.draw()
        self.sid = self.canvas.mpl_connect('pick_event', self.clickOnParentLine)
        childsids = []
        for childline,i in zip(self.childlines,range(self.numGates)):
            childsids.append(self.canvas.mpl_connect('pick_event', self.clickOnChildLines))
        self.childsids = childsids
        
        #Add title
        if groupVariable == 'Condition-Time':
            plt.title(titleVariable)
        else:
            if not isinstance(currentLevelValues, (tuple,)):
                currentLevelValues = [str(currentLevelValues)]
            plt.title('-'.join(currentLevelValues)+' ('+titleVariable+' '+str(row+1)+'/'+str(logicleDataUnstacked.shape[0])+')')
        
        def collectInputs():
            for names in range(len(pd.unique(plottingDf[groupVariable]))):
                groupedGateList.append(self.getAllX())
            with open('misc'+dirSep+'groupedGateList.pkl','wb') as f:
                pickle.dump(groupedGateList,f)
            row = pickle.load(open('misc'+dirSep+'rowValIterationRange.pkl','rb'))
            row+=1
            with open('misc'+dirSep+'rowValIterationRange.pkl','wb') as f:
                pickle.dump(row,f)
            if row != pickle.load(open('misc'+dirSep+'iterationRange.pkl','rb')):
                master.switch_frame(Unsplit_Proliferation_Gating_GUI)
            else:
                i = 0
                row = 0
                singleCellProliferationDf = pd.DataFrame(np.zeros(logicleDataStacked.shape),logicleDataStacked.index,columns=['Generation'])
                for outerVariable in outerVariableValues:
                    for innerVariable in innerVariableValues:
                        sampleGenerationGates = groupedGateList[i]
                        #sampleGenerationGates = groupedGate.getAllX()
                        if groupVariable == 'Condition':
                            indexingTuple = tuple(list(innerVariable)+[outerVariable,slice(None)])
                        else:
                            indexingTuple = tuple(list(outerVariable)+[innerVariable,slice(None)])
                        ctvValues = logicleDataStacked.loc[indexingTuple]
                        generationValues = np.zeros(ctvValues.shape)
                        for sampleEvent,row in zip(ctvValues,range(ctvValues.shape[0])):
                            for generation in range(len(sampleGenerationGates)-1):
                                upperGate = sampleGenerationGates[generation]
                                lowerGate = sampleGenerationGates[generation+1]
                                if(sampleEvent > lowerGate and sampleEvent <= upperGate):
                                    generationValues[row] = generation
                        singleCellProliferationDf.loc[indexingTuple,:] = generationValues

                print(singleCellProliferationDf)
                with open('outputData'+dirSep+'pickleFiles'+dirSep+'singleCellDataFrame-proliferation-'+folderName+'.pkl', 'wb') as f:
                    pickle.dump(singleCellProliferationDf,f)
                master.switch_frame(secondaryhomepage,folderName,expNum,ex_data,bPage)

        buttonWindow = tk.Frame(self)
        buttonWindow.pack(side=tk.TOP,pady=10)

        tk.Button(buttonWindow, text="OK",command=lambda: collectInputs()).grid(row=5,column=0)
        tk.Button(buttonWindow, text="Quit",command=quit).grid(row=5,column=2)

    def clickOnParentLine(self, event):
        if event.artist == self.line:
            #print("line selected ", event.artist)
            self.follower = self.canvas.mpl_connect("motion_notify_event", self.followParentMouse)
            self.releaser = self.canvas.mpl_connect("button_press_event", self.releaseParentOnClick)
    
    def clickOnChildLines(self, event):
        if event.artist in self.childlines:
            #print("line selected ", event.artist)
            self.currentArtist = event.artist
            self.follower = self.canvas.mpl_connect("motion_notify_event", self.followChildMouse)
            self.releaser = self.canvas.mpl_connect("button_press_event", self.releaseChildOnClick)
    
    def followChildMouse(self, event):
        for childline,gate in zip(self.childlines,range(self.numGates)):
            if self.childlines[gate] == self.currentArtist:
                self.childlines[gate].set_xdata([event.xdata, event.xdata])
        self.canvas.draw()
        #self.c.draw_idle()
    
    def releaseChildOnClick(self, event):
        newGates = []
        for childline in self.childlines:
            newGates.append(childline.get_xdata()[0])
        self.gates = newGates
        self.canvas.mpl_disconnect(self.releaser)
        self.canvas.mpl_disconnect(self.follower)

    def followParentMouse(self, event):
        self.line.set_xdata([event.xdata, event.xdata])
        newGates = []
        for childline,gate in zip(self.childlines,range(self.numGates)):
            newX = (event.xdata-self.X)+self.gates[gate]
            childline.set_xdata([newX,newX])
            newGates.append(newX)
        self.gateVals = newGates
        self.canvas.draw()
    
    def releaseParentOnClick(self, event):
        self.X = self.line.get_xdata()[0]
        newGates = []
        for childline in self.childlines:
            newGates.append(childline.get_xdata()[0])
        self.gates = newGates
        self.canvas.mpl_disconnect(self.releaser)
        self.canvas.mpl_disconnect(self.follower)

    def getParentX(self):
        return self.X
    
    def getChildX(self):
        childGates = []
        for childline in self.childlines:
            childGates.append(childline.get_xdata()[0])
        return childGates

    def getAllX(self):
        return [self.getParentX()]+self.getChildX()

def generateBulkProliferationStatistics(folderName,experimentNumber):
    singleCellProliferationDfStacked = pickle.load(open('outputData'+dirSep+'pickleFiles'+dirSep+'singleCellDataFrame-proliferation-'+folderName+'.pkl','rb'))
    idx = pd.IndexSlice
    numGenerations = len(singleCellProliferationDfStacked.loc[:,'Generation'].unique())
    singleCellProliferationDfUnstacked = singleCellProliferationDfStacked.unstack('Event')
    #Reindex unstacked dataframe by original ordering
    #conditionLevelNames = singleCellProliferationDfStacked.index.names[:-1]
    #indexDf = singleCellProliferationDfStacked.groupby(level=conditionLevelNames,sort=False).first()
    #print(indexDf)
    #indexDfFull = pickle.load(open('outputData/pickleFiles/cellStatisticPickleFile-'+folderName+'.pkl','rb'))
    firstTimepoint = singleCellProliferationDfStacked.index.unique('Time').tolist()[0]
    reindex = singleCellProliferationDfStacked.xs((1,firstTimepoint),level=('Event','Time')).index
    indexDf = singleCellProliferationDfStacked.xs((1),level=('Event')).unstack('Time').reindex(reindex).loc[:,'Generation']
    #indexDf = indexDfFull.loc[list(pd.unique(indexDfFull.index.get_level_values(0)))[0]].loc[list(pd.unique(indexDfFull.index.get_level_values(1)))[0]].loc[list(pd.unique(indexDfFull.index.get_level_values(2)))[0]]
    levelValues = []
    iteration = 0
    #temp = indexDfFull.reset_index()
    temp = singleCellProliferationDfUnstacked.reset_index()
    emptyMatrixLogicle = np.zeros(singleCellProliferationDfUnstacked.shape)
    for row in range(indexDf.shape[0]):
        levelName = list(indexDf.iloc[row,:].name)
        for timepoint in list(pd.unique(indexDf.columns)):
            tp = tuple(levelName+[timepoint,slice(None)])
            emptyMatrixLogicle[iteration,:] = singleCellProliferationDfUnstacked.loc[tp,slice(None)]
            levelValues.append(tp[:-1])
            iteration+=1
    multiIndex = pd.MultiIndex.from_tuples(levelValues, names=indexDf.index.names+[indexDf.columns.name])
    
    singleCellProliferationDfUnstacked = pd.DataFrame(emptyMatrixLogicle,index=multiIndex,columns=singleCellProliferationDfUnstacked.columns)
    singleCellProliferationDfUnstacked.columns.name = 'Event'
    #Frequency
    generationFrequencyMatrix = np.zeros([singleCellProliferationDfUnstacked.shape[0],numGenerations])
    for row in range(singleCellProliferationDfUnstacked.shape[0]):
        proliferationDfPerCondition = singleCellProliferationDfUnstacked.iloc[row,:]
        freqTable = pd.value_counts(proliferationDfPerCondition.values.ravel()).sort_index()
        generations = [float(i) for i in list(freqTable.index.get_level_values(0))]
        for generation in generations:
            generationFrequencyMatrix[row,int(generation)] = freqTable[generation]
    generationFrequencyDataFrame = pd.DataFrame(generationFrequencyMatrix,index=singleCellProliferationDfUnstacked.index,columns=range(numGenerations)).astype(int)
    generationFrequencyDataFrame.columns.name = 'Generation'
    
    fractionDilutedMatrix = np.zeros([singleCellProliferationDfUnstacked.shape[0],1])
    precursorFrequencyMatrix = np.zeros([singleCellProliferationDfUnstacked.shape[0],1])
    proliferationIndexMatrix = np.zeros([singleCellProliferationDfUnstacked.shape[0],1])
    divisionIndexMatrix = np.zeros([singleCellProliferationDfUnstacked.shape[0],1])
    for row in range(generationFrequencyDataFrame.shape[0]):
        precursorFrequencyNumerator = 0
        proliferationIndexNumerator = 0
        for generation in range(1,numGenerations):
            precursorFrequencyNumerator+=generationFrequencyDataFrame.iloc[row,generation]/(2**generation)
            proliferationIndexNumerator+=generation*generationFrequencyDataFrame.iloc[row,generation]/(2**generation)
        precursorFrequencyDenominator = 0
        for generation in range(numGenerations):
            precursorFrequencyDenominator+=generationFrequencyDataFrame.iloc[row,generation]/(2**generation)
        
        fractionDiluted = np.sum(generationFrequencyDataFrame.iloc[row,1:])/np.sum(generationFrequencyDataFrame.iloc[row,0:])
        precursorFrequency = precursorFrequencyNumerator/precursorFrequencyDenominator
        proliferationIndex = proliferationIndexNumerator/precursorFrequencyNumerator
        divisionIndex = proliferationIndexNumerator/precursorFrequencyDenominator

        fractionDilutedMatrix[row,0] = fractionDiluted
        precursorFrequencyMatrix[row,0] = precursorFrequency
        proliferationIndexMatrix[row,0] = proliferationIndex
        divisionIndexMatrix[row,0] = divisionIndex
    allProliferationStatisticsList = []
    allProliferationMatricesList = [fractionDilutedMatrix,precursorFrequencyMatrix,proliferationIndexMatrix,divisionIndexMatrix]
    allProliferationStatisticNames = ['fractionDiluted','precursorFrequency','proliferationIndex','divisionIndex']
    for i in range(generationFrequencyDataFrame.shape[1]):
        currentGenerationFrequency = np.array(generationFrequencyDataFrame.iloc[:,i].values.ravel())
        allProliferationStatisticNames.append('rawFrequencyGen'+str(i))
        allProliferationMatricesList.append(currentGenerationFrequency)
        allProliferationStatisticNames.append('normalizedFrequencyGen'+str(i))
        allProliferationMatricesList.append(np.divide(currentGenerationFrequency,2**i))
    for i in range(len(allProliferationMatricesList)):
        statisticDf = pd.DataFrame(allProliferationMatricesList[i],index=singleCellProliferationDfUnstacked.index)#,columns = [allProliferationStatisticNames[i]])
        statisticDfUnstacked = statisticDf.unstack('Time')
        #indexDfFull = pickle.load(open('outputData/pickleFiles/cellStatisticPickleFile-'+folderName+'.pkl','rb'))
        #indexDf = indexDfFull.loc[list(pd.unique(indexDfFull.index.get_level_values(0)))[0]].loc[list(pd.unique(indexDfFull.index.get_level_values(1)))[0]].loc[list(pd.unique(indexDfFull.index.get_level_values(2)))[0]]
        levelValues = []
        iteration = 0
        emptyMatrixLogicle = np.zeros(statisticDfUnstacked.shape)
        for row in range(indexDf.shape[0]):
            levelName = list(indexDf.iloc[row,:].name)
            tp = tuple(levelName)
            emptyMatrixLogicle[iteration,:] = statisticDfUnstacked.loc[tp,:]
            levelValues.append(tp)
            iteration+=1
        multiIndex = pd.MultiIndex.from_tuples(levelValues, names=indexDf.index.names)
        
        statisticDfUnstacked = pd.DataFrame(emptyMatrixLogicle,index=multiIndex,columns=statisticDfUnstacked.columns)
        statisticDfUnstacked.columns.name = 'Time'
        allProliferationStatisticsList.append(statisticDfUnstacked)
    completeBulkProliferationDataFrame = pd.concat(allProliferationStatisticsList,keys=allProliferationStatisticNames,names=['Statistic'],axis=0)
    completeBulkProliferationDataFrame.columns = completeBulkProliferationDataFrame.columns.droplevel(0)
    return completeBulkProliferationDataFrame



#! /usr/bin/env python3
from setuptools import setup,find_packages
    
install_requires = [
'matplotlib<=3.5.2',
'pandas<2.0.0',
'seaborn',
'scipy',
'openpyxl',
'tables',
'pillow<=9.5.0',
'colorcet',
'importlib_metadata'
]

classifiers = [
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    'Operating System :: Unix',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows',
    'Intended Audience :: Science/Research'
]

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="plateypus",
    version="0.6.10",
    author = "Sooraj Achar",
    author_email = "acharsr@nih.gov",
    description = "Processes and plots high throughput cytometry experiments through a GUI",
    long_description = long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/soorajachar/plateypus",
    classifiers = classifiers,
    packages=find_packages(),
    install_requires=install_requires,
    include_package_data=True,
    entry_points={
        "console_scripts": [
            "plateypus = plateypus.__main__:main"
        ]
    },
)
